==> evm_decoder-0.3.5-py3-none-any.whl: evm_decoder/utils/abi_utils.py <==
import json
from eth_abi.codec import ABICodec
from eth_abi.registry import registry
from eth_utils import function_signature_to_4byte_selector, event_abi_to_log_topic, decode_hex
from typing import Dict, Any, List, Tuple
from web3 import Web3
from hexbytes import HexBytes
from web3.datastructures import AttributeDict
from .data_structures import IndexableEventLog
from .constants import UNI_V2_SWAP_TOPIC, UNI_V3_SWAP_TOPIC
abi_codec = ABICodec(registry)
w3 = Web3()

def load_abi(abi_path: str) -> List[Dict[str, Any]]:
    """Load ABI from a JSON file"""
    try:
        with open(abi_path, 'r') as f:
            return json.load(f)
    except Exception as e:
        raise ValueError(f"Failed to load ABI from {abi_path}: {str(e)}")

def get_function_selector(function_signature: str) -> str:
    return "0x" + function_signature_to_4byte_selector(function_signature).hex()

def get_event_topic(event_abi: Dict[str, Any]) -> str:
    return "0x" + event_abi_to_log_topic(event_abi).hex()

def decode_abi(types: List[str], data: str) -> List[Any]:
    return list(abi_codec.decode(types, bytes.fromhex(data[2:] if data.startswith('0x') else data)))

def decode_single(type_str: str, data: str) -> Any:
    return abi_codec.decode(type_str, decode_hex(data))
    
def encode_abi(types: List[str], values: List[Any]) -> str:
    return "0x" + abi_codec.encode(types, values).hex()

def encode_single(type_str: str, value: Any) -> str:
    return "0x" + abi_codec.encode_single(type_str, value).hex()

def decode_function_input(abi: List[Dict[str, Any]], input_data: str) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    selector = input_data[:10]
    for item in abi:
        if item['type'] == 'function' and get_function_selector(item['name']) == selector:
            types = [input['type'] for input in item['inputs']]
            names = [input['name'] for input in item['inputs']]
            decoded = abi_codec.decode(types, input_data[10:])
            return item, dict(zip(names, decoded))
    raise ValueError("Function not found in ABI")

def decode_input(params: List[Dict[str, Any]], input_data: str) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    data = input_data[10:]
    types = [param['type'] if isinstance(param, dict) else param for param in params]
    names = [param['name'] for param in params]
    try:
        decoded = abi_codec.decode(types, bytes.fromhex(data))
    except Exception as e:
        print(e)
    return dict(zip(names, decoded))


def decode_event_log(abi: List[Dict[str, Any]], log: Dict[str, Any]) -> Dict[str, Any]:
    topic0 = log['topics'][0] if 'topics' in log else log['topic0']
    # Convert topic0 to string if it's HexBytes
    if isinstance(topic0, HexBytes):
        topic0 = "0x" + topic0.hex()

    for event in abi:
        if event['type'] == 'event':
            event_id = get_event_topic(event)
            if event_id == topic0:
                decoded_log = decode_log(event['inputs'], log)
                return {
                    'decoder': 'abi_json',
                    'event': event['name'],
                    'args': decoded_log
                }
    
    raise ValueError(f"Event with topic {topic0} not found in ABI")

def decode_log(params: List[Dict[str, Any]], log: Dict[str, Any]) -> Dict[str, Any]:
    indexed_params = [p for p in params if p['indexed']]
    non_indexed_params = [p for p in params if not p['indexed']]
    decoded_topics = []
    if indexed_params:
        indexed_types = [param['type'] for param in indexed_params]
        topics = log['topics'][1:] if ('topics' in log and len(log['topics']) > 1) else [t for t in [log.get('topic1'), log.get('topic2'), log.get('topic3')] if t is not None]
        # topics = log['topics'][1:] if len(log['topics']) > 1 else [log.get('topic1'), log.get('topic2'), log.get('topic3')]
        concatenated_topics = ''.join(topic[2:] for topic in topics)  # Remove '0x' prefix and concatenate
        decoded_topics = list(abi_codec.decode(indexed_types, bytes.fromhex(concatenated_topics)))
        # print(f"decoded_topics: {decoded_topics}")
    decoded_data = []
    if non_indexed_params:
        non_indexed_types = [p['type'] for p in non_indexed_params]
        decoded_data = list(abi_codec.decode(non_indexed_types, bytes.fromhex(log['data'][2:])))
    param_names = {p['name']: i for i, p in enumerate(params) if 'name' in p}
    indexed_args = IndexableEventLog(decoded_topics, decoded_data, param_names)

    return indexed_args

def is_pair_swap(pair_address: str, data: Any) -> bool:
    return data['address'] == pair_address and (data['topic'][0] == UNI_V2_SWAP_TOPIC or data['topic'][0] == UNI_V3_SWAP_TOPIC)

def convert_hexbytes(data):
    if isinstance(data, HexBytes):
        return "0x" + data.hex()
    elif isinstance(data, (list, tuple)):
        return [convert_hexbytes(item) for item in data]
    elif isinstance(data, dict):
        return {key: convert_hexbytes(value) for key, value in data.items()}
    elif isinstance(data, AttributeDict):
        return AttributeDict({key: convert_hexbytes(value) for key, value in data.items()})
    return data

==> evm_decoder-0.3.7-py3-none-any.whl: evm_decoder/utils/abi_utils.py <==
import json
from eth_abi.codec import ABICodec
from eth_abi.registry import registry
from eth_utils import function_signature_to_4byte_selector, event_abi_to_log_topic, decode_hex
from typing import Dict, Any, List, Tuple
from web3 import Web3
from hexbytes import HexBytes
from web3.datastructures import AttributeDict
from .data_structures import IndexableEventLog
from .constants import UNI_V2_SWAP_TOPIC, UNI_V3_SWAP_TOPIC
abi_codec = ABICodec(registry)
w3 = Web3()

def load_abi(abi_path: str) -> List[Dict[str, Any]]:
    """Load ABI from a JSON file"""
    try:
        with open(abi_path, 'r') as f:
            return json.load(f)
    except Exception as e:
        raise ValueError(f"Failed to load ABI from {abi_path}: {str(e)}")

def get_function_selector(function_signature: str) -> str:
    return "0x" + function_signature_to_4byte_selector(function_signature).hex()

def get_event_topic(event_abi: Dict[str, Any]) -> str:
    return "0x" + event_abi_to_log_topic(event_abi).hex()

def decode_abi(types: List[str], data: str) -> List[Any]:
    return list(abi_codec.decode(types, bytes.fromhex(data[2:] if data.startswith('0x') else data)))

def decode_single(type_str: str, data: str) -> Any:
    return abi_codec.decode(type_str, decode_hex(data))
    
def encode_abi(types: List[str], values: List[Any]) -> str:
    return "0x" + abi_codec.encode(types, values).hex()

def encode_single(type_str: str, value: Any) -> str:
    return "0x" + abi_codec.encode_single(type_str, value).hex()

def decode_function_input(abi: List[Dict[str, Any]], input_data: str) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    selector = input_data[:10]
    for item in abi:
        if item['type'] == 'function' and get_function_selector(item['name']) == selector:
            types = [input['type'] for input in item['inputs']]
            names = [input['name'] for input in item['inputs']]
            decoded = abi_codec.decode(types, input_data[10:])
            return item, dict(zip(names, decoded))
    raise ValueError("Function not found in ABI")

def decode_input(params: List[Dict[str, Any]], input_data: str) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    data = input_data[10:]
    types = [param['type'] if isinstance(param, dict) else param for param in params]
    names = [param['name'] for param in params]
    try:
        decoded = abi_codec.decode(types, bytes.fromhex(data))
    except Exception as e:
        print(e)
    return dict(zip(names, decoded))


def decode_event_log(abi: List[Dict[str, Any]], log: Dict[str, Any]) -> Dict[str, Any]:
    topic0 = log['topics'][0] if 'topics' in log else log['topic0']
    # Convert topic0 to string if it's HexBytes
    if isinstance(topic0, HexBytes):
        topic0 = "0x" + topic0.hex()

    for event in abi:
        if event['type'] == 'event':
            event_id = get_event_topic(event)
            if event_id == topic0:
                decoded_log = decode_log(event['inputs'], log)
                return {
                    'decoder': 'abi_json',
                    'event': event['name'],
                    'args': decoded_log
                }
    
    raise ValueError(f"Event with topic {topic0} not found in ABI")

def decode_log(params: List[Dict[str, Any]], log: Dict[str, Any]) -> Dict[str, Any]:
    indexed_params = [p for p in params if p['indexed']]
    non_indexed_params = [p for p in params if not p['indexed']]
    decoded_topics = []
    if indexed_params:
        indexed_types = [param['type'] for param in indexed_params]
        topics = log['topics'][1:] if ('topics' in log and len(log['topics']) > 1) else [t for t in [log.get('topic1'), log.get('topic2'), log.get('topic3')] if t is not None]
        concatenated_topics = ''.join(topic[2:] for topic in topics)  # Remove '0x' prefix and concatenate
        decoded_topics = list(abi_codec.decode(indexed_types, bytes.fromhex(concatenated_topics)))

    decoded_data = []
    if non_indexed_params:
        non_indexed_types = [p['type'] for p in non_indexed_params]
        decoded_data = list(abi_codec.decode(non_indexed_types, bytes.fromhex(log['data'][2:])))

    # Create param_names mapping for both indexed and non-indexed parameters
    param_names = {}
    for i, param in enumerate(indexed_params):
        param_names[param['name']] = i
    for i, param in enumerate(non_indexed_params):
        param_names[param['name']] = i + len(indexed_params)

    return IndexableEventLog(decoded_topics, decoded_data, param_names)

def is_pair_swap(pair_address: str, data: Any) -> bool:
    return data['address'] == pair_address and (data['topic'][0] == UNI_V2_SWAP_TOPIC or data['topic'][0] == UNI_V3_SWAP_TOPIC)

def convert_hexbytes(data):
    if isinstance(data, HexBytes):
        return "0x" + data.hex()
    elif isinstance(data, (list, tuple)):
        return [convert_hexbytes(item) for item in data]
    elif isinstance(data, dict):
        return {key: convert_hexbytes(value) for key, value in data.items()}
    elif isinstance(data, AttributeDict):
        return AttributeDict({key: convert_hexbytes(value) for key, value in data.items()})
    return data
